import 'package:flutter/material.dart';

import 'package:sudoku/provider/data.dart';
import 'package:sudoku/utils/game_utils.dart';

class Parameters {
  static const double separatorHeight = 2.0;
  static const double blockMargin = 3.0;
  static const double blockPadding = 2.0;
  static const Color buttonBackgroundColor = Colors.white;
  static const Color buttonBorderColorActive = Colors.blue;
  static const Color buttonBorderColorInactive = Colors.white;
  static const double buttonBorderWidth = 10.0;
  static const double buttonBorderRadius = 8.0;
  static const double buttonPadding = 0.0;
  static const double buttonMargin = 0.0;

  static Widget buildParametersSelector(Data myProvider) {
    List<Widget> lines = [];

    List<String> parameters = myProvider.availableParameters;
    for (int index = 0; index < parameters.length; index++) {
      lines.add(buildParameterSelector(myProvider, parameters[index]));
      lines.add(const SizedBox(height: separatorHeight));
    }

    myProvider.loadCurrentSavedState();
    Widget buttonsBlock = myProvider.hasCurrentSavedState()
        ? buildResumeGameButton(myProvider)
        : buildStartNewGameButton(myProvider);

    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        const SizedBox(height: separatorHeight),
        Expanded(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            mainAxisAlignment: MainAxisAlignment.center,
            children: lines,
          ),
        ),
        const SizedBox(height: separatorHeight),
        Container(
          child: buttonsBlock,
        ),
      ],
    );
  }

  static Image buildImageWidget(String imageAssetCode) {
    return Image(
      image: AssetImage('assets/icons/$imageAssetCode.png'),
      fit: BoxFit.fill,
    );
  }

  static Container buildImageContainerWidget(String imageAssetCode) {
    return Container(
      child: buildImageWidget(imageAssetCode),
    );
  }

  static Column buildDecorationImageWidget() {
    return Column(
      children: [
        TextButton(
          child: buildImageContainerWidget('placeholder'),
          onPressed: () {},
        ),
      ],
    );
  }

  static Container buildStartNewGameButton(Data myProvider) {
    return Container(
      margin: const EdgeInsets.all(blockMargin),
      padding: const EdgeInsets.all(blockPadding),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              buildDecorationImageWidget(),
              Column(
                children: [
                  TextButton(
                    child: buildImageContainerWidget('button_start'),
                    onPressed: () => GameUtils.startNewGame(myProvider),
                  ),
                ],
              ),
              buildDecorationImageWidget(),
            ],
          ),
        ],
      ),
    );
  }

  static Container buildResumeGameButton(Data myProvider) {
    return Container(
      margin: const EdgeInsets.all(blockMargin),
      padding: const EdgeInsets.all(blockPadding),
      child: Table(
        defaultColumnWidth: const IntrinsicColumnWidth(),
        children: [
          TableRow(
            children: [
              Column(
                children: [
                  TextButton(
                    child: buildImageContainerWidget('button_delete_saved_game'),
                    onPressed: () => GameUtils.deleteSavedGame(myProvider),
                  ),
                ],
              ),
              Column(
                children: [
                  TextButton(
                    child: buildImageContainerWidget('button_resume_game'),
                    onPressed: () => GameUtils.resumeSavedGame(myProvider),
                  ),
                ],
              ),
              buildDecorationImageWidget(),
            ],
          ),
        ],
      ),
    );
  }

  static Widget buildParameterSelector(Data myProvider, String parameterCode) {
    List<String> availableValues = myProvider.getParameterAvailableValues(parameterCode);

    if (availableValues.length == 1) {
      return const SizedBox(height: 0.0);
    }

    return Table(
      defaultColumnWidth: const IntrinsicColumnWidth(),
      children: [
        TableRow(
          children: [
            for (int index = 0; index < availableValues.length; index++)
              Column(
                children: [
                  _buildParameterButton(myProvider, parameterCode, availableValues[index])
                ],
              ),
          ],
        ),
      ],
    );
  }

  static Widget _buildParameterButton(
      Data myProvider, String parameterCode, String parameterValue) {
    String currentValue = myProvider.getParameterValue(parameterCode).toString();

    bool isActive = (parameterValue == currentValue);
    String imageAsset = '${parameterCode}_$parameterValue';

    return TextButton(
      child: Container(
        margin: const EdgeInsets.all(buttonMargin),
        padding: const EdgeInsets.all(buttonPadding),
        decoration: BoxDecoration(
          color: buttonBackgroundColor,
          borderRadius: BorderRadius.circular(buttonBorderRadius),
          border: Border.all(
            color: isActive ? buttonBorderColorActive : buttonBorderColorInactive,
            width: buttonBorderWidth,
          ),
        ),
        child: buildImageWidget(imageAsset),
      ),
      onPressed: () => myProvider.setParameterValue(parameterCode, parameterValue),
    );
  }
}
